# Repository path:    $HeadURL: https://90.145.156.222/DMS-Houtribdijk/code/python-ftp-svn/source/misc/ReadMetaData.py $
# Last committed:     $Revision: 394466 $; $Author: ahove $; $Date: 2019-10-29 10:02:15 +0100 (di, 29 okt 2019) $
#

"""
Andere versies van dit script staan op locaties:
    C:\dms\code\nc-to-thredds\source\ReadMetaDataAdjusted_NoLog.py
    C:\fews\clients\oc\Modules\Waterlevels\waterlevels\ReadMetaDataAdjusted.py
"""


import numpy as np
import pandas as pd
import re
from misc.Utils import Utils

class ReadMetaData:

    def __init__(self, fn, ts):
        self.fn = fn
        self.ts = ts
        self.current_meta_data = {}
        self.parse_file()

    def insert_tuple(self, k, t, v):
        if k not in self.current_meta_data:
            self.current_meta_data[k] = {t: v}
        else:
            tmpTup = self.current_meta_data[k]
            tmpTup[t] = v
            self.current_meta_data[k] = tmpTup

    def parse_file(self):
        xls = pd.ExcelFile(self.fn)

        for sheet in xls.sheet_names:
            df = pd.read_excel(self.fn, sheet_name=sheet)
            df = df[df.Item.notna()]
            df['NAP Z'] = df['NAP Z'].map(lambda x: np.nan if isinstance(x, str) else x)
            # print("{}: {}".format(sheet, df.columns))

            if not sheet=="BUOY1" :
                regex='^(FL\d{2})(\w?)'
            else:
                regex='^(BUOY\d{1})(\w?)'
            locatie = Utils.get_location_from_filename(sheet, regex)

            # dataframe of the input parameter ts (timestamps)
            df_datum = pd.DataFrame(data={'Datum': self.ts})

            df_bordes1 = df[df.Item.str.match("^Bordes$", flags=re.IGNORECASE)]
            if not df_bordes1.empty:
                df_bordes = pd.merge_asof(left=df_datum, right=df_bordes1[['Datum', 'RD X', 'RD Y']], on='Datum', direction='backward')
                df_bordes.fillna(df_bordes1.iloc[0][['RD X', 'RD Y']], inplace=True)
                x = df_bordes['RD X'].values[0]
                y = df_bordes['RD Y'].values[0]
                name = df_bordes1.iloc[0]['Locatie']
                # print("df_bordes {}: {}, {}".format(locatie, x, y))
                self.insert_tuple(locatie, 'X', x)
                self.insert_tuple(locatie, 'Y', y)
                self.insert_tuple(locatie, 'Naam', name)

            df_alt1 = df[(df.Item.str.match("^ALT", flags=re.IGNORECASE))]
            df_alt1[df_alt1['NAP Z'].notnull()]
            if not df_alt1.empty:
                df_alt = pd.merge_asof(left=df_datum, right=df_alt1[['Datum', 'NAP Z']], on='Datum', direction='backward')
                df_alt.fillna(df_alt1.iloc[0]['NAP Z'], inplace=True)
                nap_z = df_alt['NAP Z'].values
                # print("df_alt {}: {}".format(locatie, nap_z))
                self.insert_tuple(locatie, 'ALT_NAP_Z', nap_z)

            df_stb1 = df[(df.Item.str.match("^STB", flags=re.IGNORECASE))]
            df_stb1[df_stb1['NAP Z'].notnull()]
            if not df_stb1.empty:
                df_stb = pd.merge_asof(left=df_datum, right=df_stb1[['Datum', 'NAP Z']], on='Datum', direction='backward')
                df_stb.fillna(df_stb1.iloc[0]['NAP Z'], inplace=True)
                nap_z = df_stb['NAP Z'].values
                # print("df_stb {}: {}".format(locatie, nap_z))
                self.insert_tuple(locatie, 'STB_NAP_Z', nap_z)

            df_adc1 = df[(df.Item.str.match("^ADC", flags=re.IGNORECASE))]
            df_adc1[df_adc1['NAP Z'].notnull()]
            if not df_adc1.empty:
                df_adc = pd.merge_asof(left=df_datum, right=df_adc1[['Datum', 'NAP Z']], on='Datum', direction='backward')
                df_adc.fillna(df_adc1.iloc[0]['NAP Z'], inplace=True)
                nap_z = df_adc['NAP Z'].values
                # print("df_adc {}: {}".format(locatie, nap_z))
                self.insert_tuple(locatie, 'ADC_NAP_Z', nap_z)

            df_adv1 = df[(df.Item.str.match("^ADV", flags=re.IGNORECASE))]
            df_adv1[df_adv1['NAP Z'].notnull()]
            if not df_adv1.empty:
                df_adv = pd.merge_asof(left=df_datum, right=df_adv1[['Datum', 'NAP Z']], on='Datum', direction='backward')
                df_adv.fillna(df_adv1.iloc[0]['NAP Z'], inplace=True)
                nap_z = df_adv['NAP Z'].values
                # print("df_adv {}: {}".format(locatie, nap_z))
                self.insert_tuple(locatie, 'ADV_NAP_Z', nap_z)

            df_adv2 = df[(df.Item.str.match("^ADV", flags=re.IGNORECASE))]
            df_adv2[df_adv2['Druk offset'].notnull()]
            if not df_adv2.empty:
                drukOffset = df_adv2['Druk offset'].values[0]
                self.insert_tuple(locatie, 'drukOffsetAdv', drukOffset)

            df_mpp1 = df[(df.Item.str.match("^MPP", flags=re.IGNORECASE))]
            df_mpp1[df_mpp1['NAP Z'].notnull()]
            if not df_mpp1.empty:
                df_mpp = pd.merge_asof(left=df_datum, right=df_mpp1[['Datum', 'NAP Z']], on='Datum', direction='backward')
                df_mpp.fillna(df_mpp1.iloc[0]['NAP Z'], inplace=True)
                nap_z = df_mpp['NAP Z'].values
                # print("df_mpp {}: {}".format(locatie, nap_z))
                self.insert_tuple(locatie, 'MPP_NAP_Z', nap_z)

            df_aqd1 = df[(df.Item.str.match("^AQD", flags=re.IGNORECASE))]
            df_aqd1[df_aqd1['NAP Z'].notnull()]
            if not df_aqd1.empty:
                df_aqd = pd.merge_asof(left=df_datum, right=df_aqd1[['Datum', 'NAP Z']], on='Datum', direction='backward')
                df_aqd.fillna(df_aqd1.iloc[0]['NAP Z'], inplace=True)
                nap_z = df_aqd['NAP Z'].values
                # print("df_aqd {}: {}".format(locatie, nap_z))
                self.insert_tuple(locatie, 'AQD_NAP_Z', nap_z)

            df_aqd2 = df[(df.Item.str.match("^AQD", flags=re.IGNORECASE))]
            df_aqd2[df_aqd2['Druk offset'].notnull()]
            if not df_aqd2.empty:
                drukOffset = df_aqd2['Druk offset'].values[0]
                self.insert_tuple(locatie, 'drukOffsetAqd', drukOffset)

    def get_meta_data(self):
        return self.current_meta_data
